<?php
/*
***********************************************************************************
DaDaBIK (DaDaBIK is a DataBase Interfaces Kreator) https://dadabik.com/
Copyright (C) 2001-2017  Eugenio Tacchini

This program is distributed "as is" and WITHOUT ANY WARRANTY, either expressed or implied, without even the implied warranties of merchantability or fitness for a particular purpose.

This program is distributed under the terms of the DaDaBIK license, which is included in this package (see dadabik_license.txt). For all the details see dadabik_license.txt.

If you are unsure about what you are allowed to do with this license, feel free to contact eugenio.tacchini@gmail.com
***********************************************************************************
*/
?>
<?php
// db functions
function connect_db($server, $user, $password, $name_db)
{
	global $debug_mode, $dbms_type, $db_schema;
	
		try {
			
			$temp = explode(':', $server);
			
			$server = $temp[0];
			$port_string = '';
			if (isset($temp[1])){
				$port_string = ';port='.$temp[1];
			}
			switch ($dbms_type){
				
				case 'sqlite':
					$conn = new PDO($dbms_type.":".$name_db, $user, $password, array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION));
					break;
				case 'mysql':
					//$conn = new PDO('mysql:host='.$server.';dbname='.$name_db, $user, $password, array(PDO::MYSQL_ATTR_INIT_COMMAND => 'SET NAMES utf8', PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION));
					
					$conn = new PDO('mysql:host='.$server.$port_string.';dbname='.$name_db, $user, $password, array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION));
					
					$res = execute_db("SET NAMES 'UTF8'", $conn);
					
					break;
				case 'postgres':
					
					$conn = new PDO('pgsql:dbname='.$name_db.$port_string.';host='.$server, $user, $password, array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION));
					
					$res = execute_db("SET NAMES 'UTF8'", $conn);
					
					$res = execute_db("SET search_path TO ".$db_schema."", $conn);

					
					break;
				default:
					echo 'Error';
					exit;
			}
  		}
		catch(PDOException $e)
    	{
    		
    		echo '<b>[06] Error:</b> during database connection. Please check $host, $user, $pass and $db_name in your config.php';
			if ($debug_mode === 1){
				echo '<br/>The DBMS server said: '.$e->getMessage();
			}
			else{
				echo ', set $debug_mode to 1 in your config.php to get further error information ';
			}
    		
    	}
    	
	return $conn;
}

function create_table_db($conn, $table_name, $fields)
{
	global $dbms_type;
	
	$sql = "CREATE TABLE ".$table_name." (".$fields;
	
	if ($dbms_type === 'mysql'){
		$sql .= ' ENGINE=InnoDB';
	}

	execute_db($sql, $conn);
}

function drop_table_db($conn, $table_name)
{
	if (table_exists($table_name)) {
		$sql = "DROP TABLE $table_name";

		execute_db($sql, $conn);
	} // end if
}

function create_index_db($conn, $data_dictionary, $table_name, $index_name, $index_fields, $options_ar)
{
	$sql_ar = $data_dictionary->CreateIndexSQL($index_name, $table_name, $index_fields, $options_ar);
	foreach ($sql_ar as $sql){
		execute_db($sql, $conn);
	} // end foreach
}

/*
function drop_index_db($conn, $data_dictionary, $table_name, $index_name)
{
	$sql_ar = $data_dictionary -> DropIndexSQL ($index_name, $table_name);
	foreach ($sql_ar as $sql){
		execute_db($sql, $conn);
	} // end foreach
}
*/

function execute_db($sql, $conn, $return_control_on_error = 0)
{
	global $debug_mode;
    	
    try {
    	$results = $conn->query($sql);
    	//$results->setFetchMode(PDO::FETCH_BOTH);
    }
    catch(PDOException $e){
    	if ($return_control_on_error === 1){
    		$res['error_message'] = $e->getMessage();
    		return $res;
    	}
    	else{
			echo '<p><b>[08] Error:</b> during query execution.';
			if ($debug_mode === 1){
				//echo ' '.htmlentities($sql).'<br/>The DBMS server said: '.$e->getMessage();
				echo ' '.htmlspecialchars($sql).'<br/>The DBMS server said: '.$e->getMessage();
			}
			else{
				echo ' Set $debug_mode to 1 in your config.php to get further error information ';
			}
			exit();
		}
    }
    
	return $results;
}

function prepare_db($conn, $sql, $return_control_on_error = 0)
{
	global $debug_mode;
    	
    try {
    	$res_prepare = $conn->prepare($sql);
    }
    catch(PDOException $e){
    	if ($return_control_on_error === 1){
    		$res_prepare['error_message'] = $e->getMessage();
    		return $res;
    	}
    	else{
			echo '<p><b>[08] Error:</b> during query preparation.';
			if ($debug_mode === 1){
				echo ' '.htmlspecialchars($sql).'<br/>The DBMS server said: '.$e->getMessage();
			}
			else{
				echo ' Set $debug_mode to 1 in your config.php to get further error information ';
			}
			exit();
		}
    }
    
	return $res_prepare;
}

function bind_param_db($res_prepare, $key, $value, $return_control_on_error = 0)
{
	global $debug_mode;
	try {
		$res_prepare->bindParam($key, $value);
	}
    catch(PDOException $e){
    	if ($return_control_on_error === 1){
    		$res_prepare['error_message'] = $e->getMessage();
    		return $res;
    	}
    	else{
			echo '<p><b>[08] Error:</b> during query execution.';
			if ($debug_mode === 1){
				echo ' '.htmlspecialchars($sql).'<br/>The DBMS server said: '.$e->getMessage();
			}
			else{
				echo ' Set $debug_mode to 1 in your config.php to get further error information ';
			}
			exit();
		}
    }
    return $res_prepare;
}

function execute_prepared_db($res_prepare, $return_control_on_error = 0)
{
	global $debug_mode;
	
	$res_execute = new stdClass();
    	
    try {
    	$res_execute = $res_prepare->execute();
    }
    catch(PDOException $e){
    	if ($return_control_on_error === 1){
    		$res_execute->error_message = $e->getMessage().' SQL Query: '.$res_prepare->queryString;
    		return $res_execute;
    	}
    	else{
			echo '<p><b>[08] Error:</b> during query execution.';
			if ($debug_mode === 1){
				echo  ' The DBMS server said: '.$e->getMessage();
			}
			else{
				echo ' Set $debug_mode to 1 in your config.php to get further error information ';
			}
			exit();
		}
    }
    return $res_execute;
}


function format_date_for_dbms($date)
{
	return "'".$date."'";
}

function format_date_time_for_dbms($date_time)
{
	return "'".$date_time."'";
}

function get_db_function_string($function, $field)
// get in input a DB function and a field to apply the function on and returns the translation for the current DBMS
// the function assumes that $field is alrady quoted
{
	global $dbms_type, $weeks_start_on_sunday;
	
	switch (strtolower($function)){
		case 'year':
			switch($dbms_type){
				case 'mysql':
					$function_string = "YEAR(".$field.")";
					break;
				case 'sqlite':
					$function_string = "strftime('%Y',".$field.")";
					break;
				case 'postgres':
					$function_string = "EXTRACT (YEAR FROM ".$field.")";
					break;
			}
			break;
		case 'month':
			switch($dbms_type){
				case 'mysql':
					$function_string = "MONTH(".$field.")";
					break;
				case 'sqlite':
					$function_string = "strftime('%m',".$field.")";
					break;
				case 'postgres':
					$function_string = "EXTRACT (MONTH FROM ".$field.")";
					break;
			}
			break;
		case 'quarter':
			switch($dbms_type){
				case 'mysql':
					$function_string = "QUARTER(".$field.")";
					break;
				case 'sqlite':
				
					$function_string = "CASE WHEN cast(strftime('%m', ".$field.") as integer) BETWEEN 1 AND 3 THEN 1 WHEN cast(strftime('%m', ".$field.") as integer) BETWEEN 4 and 6 THEN 2  WHEN cast(strftime('%m', ".$field.") as integer) BETWEEN 7 and 9 THEN 3  ELSE 4 END";
  
					break;
				case 'postgres':
					$function_string = "EXTRACT (QUARTER FROM ".$field.")";
					break;
			}
			break;
		case 'week':
			switch($dbms_type){
				case 'mysql':
					$function_string = "WEEK(".$field.")";
					break;
				case 'sqlite':
					$function_string = "strftime('%W',".$field.")";
					break;
				case 'postgres':
					$function_string = "EXTRACT (WEEK FROM ".$field.")";
					break;
			}
			break;
		case 'dayofweek':
			switch($dbms_type){
				case 'mysql':
					if ($weeks_start_on_sunday === 1){
						$function_string = "DAYOFWEEK(".$field.")";
					}
					else{
						$function_string = "WEEKDAY(".$field.")+1";
					}
					break;
				case 'sqlite':
				
					if ($weeks_start_on_sunday === 1){
						$function_string = "strftime('%w',".$field.")+1";
					}
					else{
						 $function_string = "CASE WHEN cast(strftime('%w', ".$field.") as integer) = 0 THEN '7' ELSE strftime('%w', ".$field.")  END";
					}
					
					
					break;
				case 'postgres':
					if ($weeks_start_on_sunday === 1){
						$function_string = "EXTRACT (DOW FROM ".$field.")+1";
					}
					else{
						$function_string = "EXTRACT (ISODOW FROM ".$field.")";
					}
					break;
			}
			break;
		case 'hour':
			switch($dbms_type){
				case 'mysql':
					$function_string = "HOUR(".$field.")";
					break;
				case 'sqlite':
					$function_string = "strftime('%H',".$field.")";
					break;
				case 'postgres':
						$function_string = "EXTRACT (HOUR FROM ".$field.")";
					break;
			}
			break;
		default:
			echo 'Error';
			exit;
	}
	
	return $function_string;
	
}

function execute_db_limit($sql, $conn, $records_page, $start_from)
{
	global $debug_mode, $dbms_type;
	
	switch($dbms_type){
		case 'mysql':
		case 'sqlite':
			$sql .= " LIMIT ".$start_from.", ".$records_page;
			break;
		case 'postgres':
			$sql .= " OFFSET ".$start_from." LIMIT ".$records_page;
			break;
		default:
			echo 'Error';
			exit;
	}
	
	try {
    	$results = $conn->query($sql);
    	//$results->setFetchMode(PDO::FETCH_BOTH);
    }
    catch(PDOException $e){
    	echo '<p><b>[08] Error:</b> during query execution.';
    	if ($debug_mode === 1){
    		//echo ' '.htmlentities($sql).'<br/>The DBMS server said: '.$e->getMessage();
    		echo ' '.htmlspecialchars($sql).'<br/>The DBMS server said: '.$e->getMessage();
    	}
    	exit();
    }
    
	return $results;
}

function fetch_row_db(&$rs)
{
	
	try {
		return $rs->fetch();
    }
    catch(PDOException $e){
    	echo '<p><b>[08] Error:</b> during record fetching.';
    	
    	exit();
    }
	
}

function get_num_rows_db($input, $use_sql=0)
{
	
	global $conn, $quote;
	
	if ($use_sql === 1){
		$sql = $input;
	}
	else{
		$sql = $input->queryString;
	}
	
	/*
	$temp_ar = explode(" FROM ", $sql);
	$sql = "SELECT COUNT(*) FROM ".$temp_ar[1];
	*/
	$pos = strpos_custom($sql, ' FROM ');
	$sql = "SELECT COUNT(*) FROM ".substr_custom($sql, $pos+6);
	
	
	$res = execute_db($sql, $conn);
	
	$row = fetch_row_db($res);
	
	return (int)($row[0]);
}

function get_last_ID_db($sequence_name = NULL)
{
	global $conn, $dbms_type;
	
	try {
		if (isset($sequence_name)){
			return $conn->lastInsertId($sequence_name);
		}
		else{
		    if ($dbms_type === 'postgres'){
		        return false; // to avoid possible errors with postgres, anyway without sequence name we can't get it
		    }
		    else{
			    return $conn->lastInsertId();
			}
		}
    }
    catch(PDOException $e){
    	echo '<p><b>[08] Error:</b> during last ID fetching.';
    	
    	exit();
    }
}


 
 
 /*
function list_fields_db($db_name, $table_name, $conn)
{
	//return mysql_list_fields ($db_name, $table_name, $conn);
	return $conn->MetaColumns($table_name);
}

*/

function num_fields_db($fields)
{
	
	try {
		return $fields->columnCount();
    }
    catch(PDOException $e){
    	echo '<p><b>[08] Error:</b> during number field fetching.';
    	
    	exit();
    }
}

function get_autoincrement_field($table_name)
// only implemented for mysql at the moment
{
	global $conn, $dbms_type, $quote;
	
	$autoincrement_field = NULL;
	
	switch ($dbms_type){
	
	    case 'mysql';
	        $sql = "show columns from ".$quote.$table_name.$quote." WHERE Extra = 'auto_increment'";
	        	
	        $res = execute_db($sql, $conn);
	        
	        while($row = fetch_row_db($res)){
	        	$autoincrement_field = $row['Field'];
	        }
	        
	        break;
	    case 'postgres';
	        break;
	    case 'sqlite';
	        break;
	}
	
	return $autoincrement_field;
	
}

function get_unique_field_db($table_name, $directly_from_db = 0, $fallback_to_db = 0)
// goal: get the name of the first uniqe field in a table
// input: $table_name, $fallback_to_db (0|1) if 1 and $directly_from_db = 0 and the table is not in the dadabik_table_list, go directly to the db
// output: $unique_field_name, the name of the first unique field in the table
{
	global $conn, $dbms_type, $quote, $table_list_name;
	
	switch($directly_from_db){
		case 0:
			$sql = "SELECT pk_field_table FROM ".$quote.$table_list_name.$quote." WHERE name_table = '".$table_name."'";
		
			$res = execute_db($sql, $conn);
			
			if ( ($row = fetch_row_db($res)) === false){
				if ($fallback_to_db === 1){
					return get_unique_field_db($table_name, 1);
				}
			}
			else{
			
				return $row['pk_field_table'];
			}
			
			break;
		case 1:
		
			switch ($dbms_type){
				case 'mysql':
					$sql = "show columns from ".$quote.$table_name.$quote;
					break;
				case 'sqlite':
					$sql = "PRAGMA table_info(".$quote.$table_name.$quote.")";
					break;
				case 'postgres':
					$sql = "SELECT pg_attribute.attname FROM pg_index, pg_class, pg_attribute WHERE pg_class.oid = '".$table_name."'::regclass AND indrelid = pg_class.oid AND pg_attribute.attrelid = pg_class.oid AND  pg_attribute.attnum = any(pg_index.indkey)";
					break;
				default:
					echo 'Error';
					exit;
			}
			
			foreach (execute_db($sql, $conn) as $row){
				switch ($dbms_type){
					case 'mysql':
						if ($row['Key'] == 'PRI'){
							return $row['Field'];
						}
						break;
					case 'sqlite':
						if ($row['pk'] == 1){
							return $row['name'];
						}
						break;
					case 'postgres':
						return $row['attname'];
						break;
					default:
						echo 'Error';
						exit;
				}
			}
			
			return NULL;
			
			
			break;
	}
	
} // end function get_unique_field_db


function get_tables_list($get_additional_info = 0)
// goal: get a list of the tables available in the current database
// input:
// output: $tables_ar, an array containing all the table names
{
	global $conn, $dbms_type, $db_schema;
	$tables_ar = array();
	
	switch ($dbms_type){
		case 'mysql':
			$sql = "SELECT TABLE_NAME as name, TABLE_TYPE as type FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA=SCHEMA() order by TABLE_NAME";
			break;
		case 'sqlite':
			$sql = "SELECT name, type FROM sqlite_master WHERE (type='table' or type = 'view') AND name <> 'sqlite_sequence' order by name";
			break;
		case 'postgres':
			$sql = "select table_name as name, 'table' as type, table_schema from information_schema.tables where table_schema not in ('pg_catalog','information_schema') union select table_name as name, 'view' as type, table_schema  from information_schema.views where table_schema not in ( 'pg_catalog','information_schema') order by name";
			break;
		default:
			echo 'Error';
			exit;
	}
	$c = 0;
	foreach (execute_db($sql, $conn) as $row){
		
		if ($dbms_type !== 'postgres' || $row['table_schema'] === $db_schema){
            if ($get_additional_info === 0){
                $tables_ar[] = $row['name'];
            }
            else{
                $tables_ar[$c]['name'] = $row['name'];
                if ($row['type'] === 'BASE TABLE'){ // for mysql
                    $row_temp = 'table';
                }
                else{
                    $row_temp = strtolower($row['type']);
                }
                $tables_ar[$c]['type'] = $row_temp;
            }
            $c++;
        }
		
	}
	
	
	return $tables_ar;
	
}

function get_fields_list($table_name, $add_additional_info = false)
// goal: get a list of the fields available in a table
// input:
// output: $fields_ar, an array containing all the field names
{
	global $conn, $dbms_type, $quote, $db_name;
	$fields_ar = array();
	
	switch ($dbms_type){
		case 'mysql':
			$sql = "show columns from ".$quote.$table_name.$quote;
			foreach (execute_db($sql, $conn) as $row){
				$fields_ar['name'][] = $row['Field'];
				$fields_ar['type'][] = $row['Type'];
				if ($row['Null'] == 'YES'){
					$fields_ar['null'][] = 1; 
				}
				else{
					$fields_ar['null'][] = 0; 
				}
				$fields_ar['extra'][] = $row['Extra'];
			}
			
			$sql = "SELECT column_name, referenced_table_name, referenced_column_name FROM information_schema.KEY_COLUMN_USAGE WHERE table_schema = '".$db_name."' and table_name = '".$table_name."' and referenced_table_schema = '".$db_name."' and referenced_column_name IS NOT NULL";
			$fks=array();
			foreach (execute_db($sql, $conn) as $row){
				$fks[$row['column_name']]['referenced_table_name'] = $row['referenced_table_name'];
				$fks[$row['column_name']]['referenced_field_name'] = $row['referenced_column_name'];
			}
			foreach($fields_ar['name'] as $key => $value){
				if (isset($fks[$value])){
					$fields_ar['referenced_table_name'][$key] = $fks[$value]['referenced_table_name'];
					$fields_ar['referenced_field_name'][$key] = $fks[$value]['referenced_field_name'];
				}
			}
			
			break;
		case 'sqlite':
			$sql = "PRAGMA table_info(".$quote.$table_name.$quote.")";
			foreach (execute_db($sql, $conn) as $row){
				$fields_ar['name'][] = $row[1];
				$fields_ar['type'][] = $row[2];
				if ($row[3] == '1'){ // sqlite has a notnull 0|1 property
					$fields_ar['null'][] = 0; 
				}
				else{
					$fields_ar['null'][] = 1; 
				}
				
				$fields_ar['extra'][] = '';
			}
			$sql = "PRAGMA foreign_key_list(".$quote.$table_name.$quote.")";
			$fks=array();
			foreach (execute_db($sql, $conn) as $row){
				$fks[$row['from']]['referenced_table_name'] = $row['table'];
				$fks[$row['from']]['referenced_field_name'] = $row['to'];
			}
			foreach($fields_ar['name'] as $key => $value){
				if (isset($fks[$value])){
					$fields_ar['referenced_table_name'][$key] = $fks[$value]['referenced_table_name'];
					$fields_ar['referenced_field_name'][$key] = $fks[$value]['referenced_field_name'];
				}
			}
			
			break;
		case 'postgres':
			$sql = "SELECT column_name, data_type, is_nullable, character_maximum_length FROM information_schema.columns WHERE table_name ='".$table_name."' order by ordinal_position";
			foreach (execute_db($sql, $conn) as $row){
				$fields_ar['name'][] = $row['column_name'];
				$fields_ar['type'][] = $row['data_type'];
				
				
				if ($row['is_nullable'] == 'YES'){
					$fields_ar['null'][] = 1; 
				}
				else{
					$fields_ar['null'][] = 0; 
				}
				$fields_ar['max_length'][] = $row['character_maximum_length'];
				$fields_ar['extra'][] = '';
			}
			break;
		default:
			echo 'Error';
			exit;
	}
	
	if ($add_additional_info === true){
		return $fields_ar;
	}
	else{
		return $fields_ar['name'];
	}
}

function escape($string)
{
	global $conn, $dbms_type;
	
	switch ($dbms_type){
		case 'mysql':
			return addslashes($string);
			break;
		case 'postgres':
			return str_replace("'", "''", $string);
			break;
		case 'sqlite':
			return str_replace("'", "''", $string);
			break;
		default:
			echo 'Error';
			exit;
	}
}

function unescape($string, $for_magic = 0)
{
	global $conn, $dbms_type;
	
	$unesape_pgsql_sqlite_from = '\\\'';
	$unesape_pgsql_sqlite_to = '\'';
	
	if (  ini_get('magic_quotes_sybase') == '1' || $for_magic == 0){
		$unesape_pgsql_sqlite_from = "''";
		$unesape_pgsql_sqlite_to = "'";
	}
	
	switch ($dbms_type){
		case 'mysql':
			if (is_array($string)){ // for select_multiple_*
				$array_new = array();
				foreach($string as $value){
					$array_new[] = stripslashes($value);
				}
				return $array_new;
			}
			return stripslashes($string);
			break;
		case 'postgres':
			if (is_array($string)){ // for select_multiple_*
				$array_new = array();
				foreach($string as $value){
					$array_new[] = str_replace($unesape_pgsql_sqlite_from, $unesape_pgsql_sqlite_to, $value);
				}
				return $array_new;
			}
			return str_replace($unesape_pgsql_sqlite_from, $unesape_pgsql_sqlite_to, $string);
			break;
		case 'sqlite':
			if (is_array($string)){ // for select_multiple_*
				$array_new = array();
				foreach($string as $value){
					$array_new[] = str_replace($unesape_pgsql_sqlite_from, $unesape_pgsql_sqlite_to, $value);
				}
				return $array_new;
			}
			return str_replace($unesape_pgsql_sqlite_from, $unesape_pgsql_sqlite_to, $string);
			break;
		default:
			echo 'Error';
			exit;
	}
}

function begin_trans_db()
{
	global $conn;
	
	try {
		$conn->beginTransaction();
    }
    catch(PDOException $e){
    	echo '<p><b>[08] Error:</b> during transaction start.';
    	
    	exit();
    }
}

function complete_trans_db()
{
	global $conn;
	
	try {
		$conn->commit();
    }
    catch(PDOException $e){
    	echo '<p><b>[08] Error:</b> during transaction commit.';
    	
    	exit();
    }
}
?>